! gtool_variable.f90 - gtool4 file interface
! Copyright (C) by TOYODA Eizi, 2000.  All rights reserved.
! vi: set sw=4 ts=8:

module gtool_variable 

    use netcdf
    use gtool_file
    use dc_string
    implicit none

    type GT_VARIABLE
	type(GT_FILE)::	gtfile
	integer:: contents
	type(NC_VARIABLE):: ncvar
	type(NC_DIMENSION):: ncdim
    end type

    integer, parameter:: GT_VARIABLE_UNKNOWN = 0
    integer, parameter:: GT_VARIABLE_NETCDF = 1
    integer, parameter:: GT_VARIABLE_NETCDF_DIM = 2
    integer, parameter:: GT_VARIABLE_NETCDF_DIMVAR = 3

    interface Contents
	module procedure GtoolVariableContents
    end interface

    interface Create
	module procedure GtoolVariableCreate
    end interface

    interface Open
	module procedure GtoolVariableOpen
    end interface

    interface Dimension
	module procedure GtoolVariableToDimension
    end interface

    interface Close
	module procedure GtoolVariableClose
    end interface

    interface Name
	module procedure GtoolVariableName
    end interface

    interface Fullname
	module procedure GtoolVariableFullname
    end interface

    interface MaxVal
	module procedure GtoolVariableMaxVal
    end interface

    interface MinVal
	module procedure GtoolVariableMinVal
    end interface

contains

    integer function GtoolVariableContents(var) result(result)
	type(GT_VARIABLE), intent(in)::		var
    continue
	result = var%contents
	if (var%contents == GT_VARIABLE_NETCDF) return
	if (var%contents == GT_VARIABLE_NETCDF_DIM) return
	if (var%contents == GT_VARIABLE_NETCDF_DIMVAR) return
	result = GT_VARIABLE_UNKNOWN
    end function

    subroutine FilenameSplit(fullname, file, var, attr)
	type(VARYING_STRING), intent(in)::	fullname
	type(VARYING_STRING), intent(out), optional::	file, var, attr
	type(VARYING_STRING):: varpart
	integer:: hash, colon
    continue
	hash = index(char(fullname), '#', back=.TRUE.)
	if (hash == 0) then
	    if (present(file)) file = fullname
	    if (present(var)) var = ''
	    if (present(attr)) attr = ''
	    return
	endif
	if (present(file)) file = extract(fullname, 1, hash - 1)
	varpart = extract(fullname, hash + 1)
	colon = index(var, ':')
	if (colon == 0) then
	    if (present(var)) var = varpart
	    if (present(attr)) attr = ''
	    varpart = ''
	    return
	endif
	if (present(var)) var = extract(varpart, 1, colon - 1)
	if (present(attr)) attr = extract(varpart, colon + 1)
	varpart = ''
    end subroutine

    subroutine GtoolVariableCreate(var, fullname, fail)
	type(GT_VARIABLE), intent(inout)::		var
	type(VARYING_STRING), intent(in)::		fullname
	type(VARYING_STRING):: 		filename, varname
	type(NC_FILE)::					ncfile
	logical, intent(out), optional:: fail
    continue
	if (present(fail)) fail = .FALSE.
	! try to open netCDF file
	call FilenameSplit(fullname, filename, varname)
	var%contents = GT_VARIABLE_NETCDF
	ncfile = NetcdfOpenFile(char(filename), writable=.TRUE.)
	if (.error. ncfile) ncfile = NetcdfCreateFile(char(fullname))
	if (.not..error. ncfile) then
	    var%ncvar = Variable(ncfile, char(varname))
	    if (.not..error. (var%ncvar)) then
		return
	    endif
	endif
	! if opening netCDF fail
	if (present(fail)) then
	    fail = .TRUE.
	else
	    call NetcdfAssert()
	endif
    end subroutine

    ! fake
    type(VARYING_STRING) function FirstVariable(file) result(result)
	type(GT_FILE), intent(in)::		file
	type(NC_FILE):: ncfile
	integer:: i
	type(NC_VARIABLE):: v
    continue
    	do, i = 1, VariablesNumber(.netcdf. file)
	    v = Variable(.netcdf. file, i)
	    result = Name(v)
	    if (.error. Dimension(.netcdf. file, char(result))) return
	enddo
	v = Variable(.netcdf. file, 1)
	result = Name(v)
    end function

    subroutine GtoolVariableOpen(var, fullname, writable, fail)
	type(GT_VARIABLE), intent(inout):: var
	type(VARYING_STRING), intent(in):: fullname
	logical, intent(in), optional:: writable
	logical, intent(out), optional:: fail
	type(VARYING_STRING):: filename, varname
	type(NC_FILE):: ncfile
    continue
	if (present(fail)) fail = .FALSE.
	! try to open netCDF file
	call FilenameSplit(fullname, filename, varname)
	var%contents = GT_VARIABLE_NETCDF
	call Open(var%gtfile, filename, fail=fail, writable=writable)
	if (present(fail)) then
	    if (fail) then
		var%contents = GT_VARIABLE_UNKNOWN;  return
	    endif
	endif
	if (varname == '') varname = FirstVariable(var%gtfile)
	var%ncvar = Variable(.netcdf. var%gtfile, char(varname))
	if (.error. var%ncvar) then
	    fail = .TRUE.
	    var%contents = GT_VARIABLE_UNKNOWN;  return
	endif
    end subroutine

    function GtoolVariableToDimension(var, dim_ord) result(result)
	type(GT_VARIABLE):: result
	type(GT_VARIABLE), intent(in):: var
	integer, intent(in):: dim_ord
	type(NC_DIMENSION), pointer:: alldims(:)
	type(NC_VARIABLE):: dimvar
    continue
	alldims => Dimensions(var%ncvar)
	dimvar = Variable(var%ncvar%file, char(Name(alldims(dim_ord))))
	result%ncdim = alldims(dim_ord)
	if (.not. .error. dimvar) then
	    result%ncvar = dimvar
	    result%contents = GT_VARIABLE_NETCDF_DIMVAR
	else
	    result%contents = GT_VARIABLE_NETCDF_DIM
	endif
	deallocate(alldims)
    end function

    subroutine GtoolVariableClose(var, fail)
	type(GT_VARIABLE), intent(inout)::		var
	logical, optional, intent(out):: fail
	logical:: myfail
    continue
	call Close(var%gtfile, fail=myfail)
	if (present(fail)) fail = myfail
	if (Contents(var) == GT_VARIABLE_NETCDF .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIMVAR .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIM) then
	    var%contents = GT_VARIABLE_UNKNOWN
	else
	    if (present(fail)) then
		fail = .TRUE.
	    else
		print "('Close: invalid GT_VARIABLE handle')"
		stop
	    endif
	endif
    end subroutine

    type(VARYING_STRING) function GtoolVariableName(var) result(result)
	type(GT_VARIABLE), intent(in):: var
    continue
	if (Contents(var) == GT_VARIABLE_NETCDF .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIMVAR) then
	    result = Name(var%ncvar)
	else if (Contents(var) == GT_VARIABLE_NETCDF_DIM) then
	    result = Name(var%ncdim)
	else
	    result = ''
	endif
    end function

    type(VARYING_STRING) function GtoolVariableFullname(var) result(result)
	type(GT_VARIABLE), intent(in):: var
    continue
	if (Contents(var) == GT_VARIABLE_NETCDF .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIMVAR) then
	    result = Fullname(var%ncvar)
    	else if (Contents(var) == GT_VARIABLE_NETCDF_DIM) then
	    result = Fullname(var%ncdim)
	else
	    result = ''
	endif
    end function

    real function GtoolVariableMaxVal(var) result(result)
	type(GT_VARIABLE), intent(in):: var
	real, pointer:: buffer(:, :, :, :, :, :, :)
    continue
	if (Contents(var) == GT_VARIABLE_NETCDF .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIMVAR) then
	    buffer => get_real(var%ncvar)
	    if (.not.associated(buffer)) then
		result = 1.0; return
	    endif
	    result = maxval(buffer)
	    deallocate(buffer)
	else if (Contents(var) == GT_VARIABLE_NETCDF_DIM) then
	    result = len(var%ncdim)
	else
	    result = 0
	endif
    end function

    real function GtoolVariableMinVal(var) result(result)
	type(GT_VARIABLE), intent(in):: var
	real, pointer:: buffer(:, :, :, :, :, :, :)
    continue
	if (Contents(var) == GT_VARIABLE_NETCDF .or. &
	    & Contents(var) == GT_VARIABLE_NETCDF_DIMVAR) then
	    buffer => get_real(var%ncvar)
	    if (.not.associated(buffer)) then
		result = 0; return
	    endif
	    result = minval(buffer)
	    deallocate(buffer)
	else if (Contents(var) == GT_VARIABLE_NETCDF_DIM) then
	    result = 1
	else
	    result = 0
	endif
    end function

end module
